package com.hai.service.impl;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.DateUtil;
import com.hai.dao.HighUserMapper;
import com.hai.entity.HighGoldRec;
import com.hai.entity.HighUser;
import com.hai.entity.HighUserExample;
import com.hai.service.HighGoldRecService;
import com.hai.service.HighUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Sum1Dream
 */
@Service("highUserServiceImpl")
public class HighUserServiceImpl implements HighUserService {

    @Resource
    private HighUserMapper highUserMapper;

    @Resource
    private HighGoldRecService highGoldRecService;

    @Override
    public List<HighUser> getListUser(Map<String, String> map) {

        HighUserExample example = new HighUserExample();
        HighUserExample.Criteria criteria = example.createCriteria();

        if (StringUtils.isNotBlank(map.get("phone"))) {
            criteria.andPhoneLike("%" + map.get("phone") + "%");
        }
        if (StringUtils.isNotBlank(map.get("name"))) {
            criteria.andNameLike("%" + map.get("name") + "%");
        }
        if (StringUtils.isNotBlank(map.get("status"))) {
            criteria.andStatusEqualTo(Integer.valueOf(map.get("status")));
        }
        if (StringUtils.isNotBlank(map.get("regTimeStart")) && StringUtils.isNotBlank(map.get("regTimeEnd"))) {
            criteria.andRegTimeBetween(
                    DateUtil.format(map.get("regTimeStart") , "yyyy-MM-dd HH:mm:ss") ,
                    DateUtil.format(map.get("regTimeEnd") , "yyyy-MM-dd HH:mm:ss"));
        }

        return highUserMapper.selectByExample(example);

    }

    @Override
    public HighUser findByUserId(Long userId) {
        return highUserMapper.selectByPrimaryKey(userId);
    }

    @Override
    public HighUser findByOpenId(String openId) {
        HighUserExample example = new HighUserExample();
        example.createCriteria().andOpenIdEqualTo(openId);
        List<HighUser> list = highUserMapper.selectByExample(example);
        if (list != null && list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public HighUser findByPhone(String phone) {
        HighUserExample example = new HighUserExample();
        example.createCriteria().andPhoneEqualTo(phone);
        List<HighUser> list = highUserMapper.selectByExample(example);
        if (list != null && list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public void updateUser(HighUser highUser) {
        highUserMapper.updateByPrimaryKeySelective(highUser);
    }

    @Override
    public void insertUser(HighUser highUser) {
        highUserMapper.insert(highUser);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void goldHandle(Long userId, Integer goldNum, Integer goldType, Integer resType, Long resId) {
        // 查询用户信息
        HighUser user = highUserMapper.selectByPrimaryKey(userId);
        if (user == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "");
        }

        // 金币类型：1收入，2支出
        if (goldType == 1) {
            user.setGold(user.getGold() + goldNum);

        } else  if (goldType == 2) {
            if (goldNum > user.getGold()) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "金币数量不足");
            }
            user.setGold(user.getGold() - goldNum);

        } else {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "用户金币处理异常");
        }

        HighGoldRec highGoldRec = new HighGoldRec();
        highGoldRec.setUserId(user.getId());
        highGoldRec.setGoldType(goldType.longValue());
        highGoldRec.setGold(goldNum);
        highGoldRec.setCreateTime(new Date());
        highGoldRec.setResType(resType);
        highGoldRec.setResId(resId);
        highGoldRecService.insertGoldRec(highGoldRec);
    }
}
